
const bc_margin = {
    top: 50,
    right: 400,
    bottom: 45,
    left: 75,
  },
  bc_width = 945 - bc_margin.left - bc_margin.right,
  bc_height = 472 - bc_margin.top - bc_margin.bottom;

// append the svg object to the body of the page
const bc_svg = d3
  .select('#bubble_chart')
  .append('svg')
  .attr(
    'width',
    bc_width + bc_margin.left + bc_margin.right,
  )
  .attr(
    'height',
    bc_height + bc_margin.top + bc_margin.bottom,
  )
  .append('g')
  .attr(
    'transform',
    `translate(${bc_margin.left},${bc_margin.top})`,
  );

// read the data
d3.csv('bubblechart.csv').then((data) => {
  // add chart title
  bc_svg
    .append('text')
    .attr('x', bc_width / 2)
    .attr('y', -20)
    .attr('text-anchor', 'middle')
    .style('font-size', '22px')
    .text('Bubble Chart for Electronics');

  // add x axis
  const x = d3
    .scaleLinear()
    .domain([30, 200])
    .range([0, bc_width]);
  bc_svg
    .append('g')
    .attr('transform', `translate(0, ${bc_height})`)
    .call(d3.axisBottom(x));

  // add x axis label
  bc_svg
    .append('text')
    .attr('text-anchor', 'middle')
    .attr('x', bc_width / 2)
    .attr('y', bc_height + 40)
    .text('Screen Size (Inches)');

  // add y axis
  const y = d3
    .scaleLinear()
    .domain([500, 13000])
    .range([bc_height, 0]);
  bc_svg.append('g').call(d3.axisLeft(y));

  // add y axis label:
  bc_svg
    .append('text')
    .attr('text-anchor', 'middle')
    .attr('x', -bc_height / 2)
    .attr('y', -50)
    .text('Price (USD)')
    .attr('transform', 'rotate(-90)');

  // add a scale for bubble size
  const z = d3
    .scaleLinear()
    .domain([
      0,
      d3.max(
        data,
        (d) => +d['Resolution(Horizontal Pixels)'],
      ),
    ])
    .range([1, 40]);

  // add dots
  bc_svg
    .append('g')
    .selectAll('dot')
    .data(data)
    .join('circle')
    .attr('class', 'bubbles')
    .attr('cx', (d) => x(d['Screen_Size(Inches)']))
    .attr('cy', (d) => y(d['Price(USD)']))
    .attr('r', (d) => z(d['Resolution(Horizontal Pixels)']))
    .style('fill', 'skyblue')
    .attr('stroke', 'black')
    .attr('opacity', 0.7);

  // ---------------------------//
  //      tooltip               //
  // ---------------------------//

  // -1- create a tooltip div that is hidden by default:
  const tooltip = d3
    .select('#bubble_chart')
    .append('div')
    .style('opacity', 0)
    .attr('class', 'tooltip')
    .style('background-color', 'black')
    .style('border-radius', '5px')
    .style('padding', '10px')
    .style('color', 'white')
    .style('font-family', 'sans-serif');

  // -2- create 3 functions to show / update (when mouse moves but stay on same circle) / hide the tooltip
  const showTooltip = (event, d) => {
    tooltip.transition().duration(200);
    tooltip
      .style('opacity', 1)
      .html(
        `Brand: ${d.Brand}<br>Screen Size: ${d['Screen_Size(Inches)']} inches<br>Resolution: ${d['Resolution(Horizontal Pixels)']} x ${d['Resolution(Vertical Pixels)']} pixels<br>Processor Speed: ${d['Processor_Speed(GHz)']} GHz<br>Price: $${d['Price(USD)']}`,
      );
  };
  const moveTooltip = (event, d) => {
    tooltip
      .style('left', event.x + 25 + 'px')
      .style('top', event.y + 25 + 'px');
  };
  const hideTooltip = (event, d) => {
    tooltip.transition().duration(200).style('opacity', 0);
  };

  // -3- Trigger the functions on mouse events
  bc_svg
    .selectAll('circle')
    .on('mouseover', showTooltip)
    .on('mousemove', moveTooltip)
    .on('mouseleave', hideTooltip);
});
